#ifndef BHN_H
#define BHN_H

#define NETBUFSIZE	2048

#define INADDR(a,b,c,d)		(((a) << 24) | ((b) << 16) | ((c) << 8 ) | (d))
#define SOCKADDR	struct sockaddr *

#define GCMDPORT(map) (map)->unit->base_port
#define CMDPORT(map) ({							\
		struct bhn_map *lmap = (map);				\
		lmap->unit->base_port + (lmap->unit_port * 2);		\
	})
#define DATAPORT(map) ({						\
		struct bhn_map *lmap = (map);				\
		lmap->unit->base_port + (lmap->unit_port * 2) + 1;	\
	})

#define WRITE_BUF_SZ	PAGE_SIZE

struct bhn_unit {
	struct bhn_info info;
	int base_port;
	int base_baud;
	struct bhn_portcap pcap;
	struct bhn_map *ports[MAX_PORTS];
	struct sockaddr_in addr;
};

/* bhn_map flags */
#define MAP_AUTO_RECONNECT	0x00000001
#define MAP_OPENING		0x00000002
#define MAP_CLOSING		0x00000004
#define MAP_OPENED		0x00000008
#define MAP_NETWORK_ALIVE	0x00000010
#define MAP_PURGING		0x00000020
#define ASYNC_THROTTLE		0x00200000

struct bhn_map {
	struct bhn_unit *unit;
	int unit_port;
	int lim;
	struct bhn_state *state;
	struct semaphore openlock;
	spinlock_t fliplock;
	int cmd_timeout;
	int flags;
	int auto_reconn_pid;
	struct semaphore auto_reconn_lock;
	struct completion auto_reconn_comp;
	struct semaphore cr_lock;
	struct semaphore cw_lock;
	struct semaphore dr_lock;
	struct semaphore dw_lock;
};

struct bhn_connection {
	struct sockaddr_in addr;
	struct socket *sock;
	int read_pid;
	struct completion read_comp;
	int write_pid;
	struct completion write_comp;
	wait_queue_head_t wq_write;
	unsigned char read_buf[MAX_NET_CMD_SZ];
	int read_count;
	unsigned char write_buf[WRITE_BUF_SZ];
	int write_head;
	int write_tail;
};

struct bhn_state {
	struct bhn_connection cmd;
	struct bhn_connection data;
	struct list_head cmdlist;
	spinlock_t listlock;
	wait_queue_head_t wq_throttle;
	wait_queue_head_t wq_purge;
	void (*event_cb)(int minor, struct bhn_event_occurred *events);
};

struct cmd_state {
	struct bhn_prot_cmd cmd;
	struct list_head list;
	struct completion reply;
	struct bhn_prot_cmd_done cmddone;
};

/*
 * Protocol functions
 */

/* Connect methods for prot_get_bhn_info */
#define CONNECT_PORT_CMD	0
#define CONNECT_PORT_GEN	1

int prot_tcp_start(int minor, void (*fn)(int minor, struct bhn_event_occurred *events));
int prot_tcp_stop(int minor);
int prot_tcp_connect(struct socket **sock, struct sockaddr *addr, int timeout);
int prot_tcp_cmdread(void *arg);
int prot_tcp_dataread(void *arg);
int prot_udp_connect(struct socket **sock);

int prot_start_autod(int port, unsigned int netaddr, unsigned char *mac,
						unsigned char *hostname);
int prot_get_autod(struct bhn_info *info, long poll, long timeout);
void prot_stop_autod(void);

int prot_ping(int minor);
int prot_open(int minor);
int prot_close(int minor);
int prot_query_tx_finished(int minor);
int prot_purge(int minor, int flags);
int prot_set_line(int minor, struct tty_struct *tty);
int prot_get_line(int minor, struct tty_struct *tty);
int prot_set_lim(int minor, int mode);
int prot_get_lim(int minor, int *mode);
int prot_get_line_state(int minor);
int prot_send_break(int minor, int time);
int prot_start_stop_break(int minor, int onoff);
int prot_set_rts(int minor, int onoff);
int prot_set_dtr(int minor, int onoff);
int prot_resume_tx(int minor);
int prot_host_type(int minor);
int prot_get_cds(int minor, struct bhn_cds_parameters *cds_para);
int prot_get_bhn_info(int minor, int which_port, struct bhn_info *bhn_info,
				unsigned char *mac, unsigned char *hostname);
int prot_get_port_info(int minor);
int prot_set_connection(int minor);
int prot_fatal_error(int minor);
int prot_port_reset(int minor);
int prot_reset_bhn(int minor);
int prot_send_immediate(int minor);
int prot_fifo_control(int minor, int rx, int tx);
int prot_set_event(int minor, int events);
int prot_get_event(int minor);
int prot_get_msr(int minor, unsigned char *msr);
int prot_send_xchar(int minor, char xchar);
int prot_set_rts_dtr(int minor, int rts_onoff, int dtr_onoff);
int prot_get_portcap(int minor);
int prot_get_mfginfo(int minor);

int receive_chars(int minor, unsigned char *s, int err);
int bhn_auto_reconnect(void *arg);

#define NPORTS		256
extern struct bhn_map bhn_table[NPORTS];
extern char *bhn_drvname;

/*
 * Network functions
 */

int send(struct socket *sock, void *buf, size_t size, unsigned flags);
int sendto(struct socket *sock, void *buf, size_t size, unsigned flags,
					struct sockaddr *addr, int addr_len);
int recv(struct socket *sock, void *buf, size_t size, unsigned flags);
int recvfrom(struct socket *sock, void *buf, size_t size, unsigned flags,
					struct sockaddr *addr, int addr_len);

/*
 * TTY-side helper functions
 */

void bhn_do_change_speed(int minor);

#endif
