/*
 * Copyright (C) 2003 Connect Tech Inc.
 *
 * Blue Heat/Net ethernet protocol definition.
 */
#ifndef CTID_PROTOCOL_H
#define CTID_PROTOCOL_H

/*
 * The windows compiler does not understand GNU's __attribute__.
 * #define it away, and set the packing alignment to 1.
 */
#ifdef _WINDOWS
  #define __attribute__(val)
  #pragma pack(1)
#endif

#define PROTOCOL_VERSION	"V6"

#define BHN_DEF_BASE_PORT	49152

				/*   Command Size	Reply Size	     */
#define PING		  1	/* g 0			0		     */
#define OPEN_PORT	  2	/* c BHN_OPEN_SZ	0		     */
#define CLOSE_PORT	  3	/* c 0			0		     */
#define TRANSMIT_DATA	  4	/* - --------------------------------------- */
#define QUERY_TX_FINISHED 5	/* c 0			0		     */
#define OUTPUT_AVAILABLE  6	/* - --------------------------------------- */
#define PURGE		  7	/* c BHN_PURGE_SZ	0		     */
#define SET_LINE	  8	/* c BHN_LSET_SZ	0		     */
#define GET_LINE	  9	/* c 0			BHN_LSET_SZ	     */
#define SET_LIM		 10	/* c BHN_LIM_SZ		0		     */
#define GET_LIM		 11	/* c 0			BHN_LIM_SZ	     */
#define GET_LINE_STATE	 12	/* c 0			BHN_LSTATE_SZ	     */
#define SEND_BREAK	 13	/* c BHN_BREAK_SZ	N/A		     */
#define START_STOP_BREAK 14	/* c BHN_SSBREAK_SZ	0		     */
#define SET_RTS		 15	/* c BHN_RTSDTR_SZ	0		     */
#define SET_DTR		 16	/* c BHN_RTSDTR_SZ	0		     */
#define GET_RTS_DTR	 17	/* - --------------------------------------- */
#define COM_LSR_EVENT	 18	/* - --------------------------------------- */
#define RESUME_TX	 19	/* c 0			N/A		     */
#define HOST_TYPE	 20	/* c 0			N/A		     */
#define GET_CDS		 21	/* g 0			BHN_CDS_PARAMETERS_SZ */
#define GET_BHN_INFO	 22	/* b BHN_INFO_CMD_SZ	BHN_INFO_SZ	     */
#define GET_PORT_INFO	 23	/* c 0			BHN_PORTINFO_SZ	     */
#define SET_CONNECTION	 24	/* c BHN_CSET_SZ	0		     */
#define FATAL_ERROR	 25	/* c 0			N/A		     */
#define BHN_RESET	 26	/* - --------------------------------------- */
#define PORT_RESET	 27	/* c 0			N/A		     */
#define RESET_BHN	 28	/* g 0			N/A		     */
#define DO_TEST		 29	/* - --------------------------------------- */
#define SEND_IMMEDIATE	 30	/* c BHN_SENDI_SZ	N/A		     */
#define FIFO_CONTROL	 31	/* c BHN_FIFO_SZ	BHN_FIFO_SZ	     */
#define SET_EVENT	 32	/* c BHN_EVENT_SZ	0		     */
#define GET_EVENT	 33	/* c 0			BHN_EVENT_SZ	     */
#define EVENT_OCCURRED	 34	/*   BHN_EVENT_OCCURRED_SZ	0	     */
#define SET_BHN_IP	 35	/* - --------------------------------------- */
#define GET_MSR		 36	/* c 0			BHN_MSR_SZ	     */
#define SEND_XCHAR	 37	/* c BHN_XCHAR_SZ	0		     */
#define SET_RTS_DTR	 38	/* c BHN_RTSANDDTR_SZ	0		     */
#define GET_PORTCAP	 39	/* b 0			BHN_PORTCAP_SZ	     */
#define GET_MFGINFO	 40	/* g 0			BHN_MFGINFO_SZ	     */

#define CMD_DONE	128
#define CMD_ACK		129
#define CMD_NAK		130

/* Command header */

#define BHN_CMD_SZ	(sizeof(struct bhn_cmd))

struct bhn_cmd {
  unsigned char cmd;
  unsigned char sequence;
  unsigned char port;
  unsigned short length;
} __attribute__((packed));

/* PING */

#define PING_CMD_SZ		(BHN_CMD_SZ)
#define PING_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

/* OPEN_PORT */

#define BHN_OPEN_SZ		(sizeof(struct bhn_open))
#define OPEN_CMD_SZ		(BHN_CMD_SZ + BHN_OPEN_SZ)
#define OPEN_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

#define OPEN_READ	0x01
#define OPEN_WRITE	0x02
#define OPEN_RDWR	0x03

struct bhn_open {
  unsigned char flags;
} __attribute__((packed));

/* CLOSE_PORT */

#define CLOSE_CMD_SZ		(BHN_CMD_SZ)
#define CLOSE_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

/* QUERY_TX_FINISHED */

#define QUERY_TX_FINISHED_CMD_SZ	(BHN_CMD_SZ)
#define QUERY_TX_FINISHED_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

/* PURGE */

#define BHN_PURGE_SZ		(sizeof(struct bhn_purge))
#define PURGE_CMD_SZ		(BHN_CMD_SZ + BHN_PURGE_SZ)
#define PURGE_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

#define PURGE_READ	0x01
#define PURGE_WRITE	0x02
#define PURGE_RDWR	0x03

struct bhn_purge {
  unsigned char flags;
} __attribute__((packed));

/* SET_LINE / GET_LINE */

#define BHN_LSET_SZ		(sizeof(struct bhn_lset))
#define SET_LINE_CMD_SZ		(BHN_CMD_SZ + BHN_LSET_SZ)
#define SET_LINE_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)
#define GET_LINE_CMD_SZ		(BHN_CMD_SZ)
#define GET_LINE_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_LSET_SZ)

#define LSET_PARITY_NONE	'n'
#define LSET_PARITY_EVEN	'e'
#define LSET_PARITY_ODD		'o'
#define LSET_PARITY_MARK	'1'
#define LSET_PARITY_SPACE	'0'

#define LSET_SFLOW_NONE		'n'
#define LSET_SFLOW_RX		'r'
#define LSET_SFLOW_TX		't'
#define LSET_SFLOW_RXTX		'b'

#define LSET_HFLOW_NONE		0x00
#define LSET_HFLOW_RTS_TOGGLE	0x01
#define LSET_HFLOW_DTR		0x02
#define LSET_HFLOW_CTS		0x08
#define LSET_HFLOW_DSR		0x10
#define LSET_HFLOW_RTS		0x80

#define LSET_SCM_NONE	0x00
#define LSET_SCM_ERROR	0x01
#define LSET_SCM_BREAK	0x02
#define LSET_SCM_EVENT	0x04
#define LSET_SCM_ESCAPE	0x08

struct bhn_lset {
  unsigned int baud;
  unsigned char databits;
  unsigned char parity;
  unsigned char stopbits;
  unsigned char sflow;
  unsigned char xoff;
  unsigned char xon;
  unsigned char hflow;
  unsigned char lloop;
  unsigned char special_char_mode;
  unsigned char error_char;
  unsigned char break_char;
  unsigned char event_char;
  unsigned char escape_char;
  unsigned char use_x_lim;
  unsigned int xoff_lim;
  unsigned int xon_lim;
} __attribute__((packed));

/* SET_LIM / GET_LIM */

#define BHN_LIM_SZ		(sizeof(struct bhn_lim))
#define SET_LIM_CMD_SZ		(BHN_CMD_SZ + BHN_LIM_SZ)
#define SET_LIM_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)
#define GET_LIM_CMD_SZ		(BHN_CMD_SZ)
#define GET_LIM_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_LIM_SZ)

#define LIM_MODE	0x07
#define LIM_UNKNOWN	0x00
#define LIM_232		0x01
#define LIM_FULLD	0x02
#define LIM_HALFD	0x03
#define LIM_SLAVE	0x04
#define LIM_TRI		0x05
#define LIM_BT		0x08

struct bhn_lim {
  unsigned char mode;
} __attribute__((packed));

/* GET_LINE_STATE */

#define BHN_LSTATE_SZ			(sizeof(struct bhn_lstate))
#define GET_LINE_STATE_CMD_SZ		(BHN_CMD_SZ)
#define GET_LINE_STATE_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_LSTATE_SZ)

#define LSTATE_CTS	0x01
#define LSTATE_DSR	0x02
#define LSTATE_RI	0x04
#define LSTATE_DCD	0x08
#define LSTATE_RTS	0x10
#define LSTATE_DTR	0x20

struct bhn_lstate {
  unsigned char lines;
} __attribute__((packed));

/* SEND_BREAK */

#define BHN_BREAK_SZ		(sizeof(struct bhn_break))
#define SEND_BREAK_CMD_SZ	(BHN_CMD_SZ + BHN_BREAK_SZ)

struct bhn_break {
  unsigned char duration;
} __attribute__((packed));

/* START_STOP_BREAK */

#define BHN_SSBREAK_SZ			(sizeof(struct bhn_ssbreak))
#define START_STOP_BREAK_CMD_SZ		(BHN_CMD_SZ + BHN_SSBREAK_SZ)
#define START_STOP_BREAK_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

#define SSBREAK_OFF	0x00
#define SSBREAK_ON	0x01

struct bhn_ssbreak {
  unsigned char onoff;
} __attribute__((packed));

/* SET_RTS / SET_DTR */

#define BHN_RTSDTR_SZ		(sizeof(struct bhn_rtsdtr))
#define SET_RTS_CMD_SZ		(BHN_CMD_SZ + BHN_RTSDTR_SZ)
#define SET_RTS_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)
#define SET_DTR_CMD_SZ		(BHN_CMD_SZ + BHN_RTSDTR_SZ)
#define SET_DTR_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

#define RTS_OFF		0x00
#define RTS_ON		0x01
#define DTR_OFF		0x00
#define DTR_ON		0x01

struct bhn_rtsdtr {
  unsigned char onoff;
} __attribute__((packed));

/* RESUME_TX */

#define RESUME_TX_CMD_SZ	(BHN_CMD_SZ)

/* HOST_TYPE */

#define HOST_TYPE_CMD_SZ	(BHN_CMD_SZ)

/* GET_CDS */

/*
 * WARNING:
 *
 * The compilers currently used to build the linux host driver, one version
 * of which is listed below, seem to always pad out the CDS_Parameters
 * struct to a four byte boundry.
 * # gcc -v
 * gcc version 2.96 20000731 (Red Hat Linux 7.1 2.96-98)
 *
 * The compiler currently used to build the firmware DOES NOT DO THIS.
 * # m68k-elf-gcc -v
 * gcc version 2.95.3 20010315 (release)(ColdFire patches - 20010318 from
 *                            http://fiddes.net/coldfire/)(-msep-data patches)
 *
 * That is to say, sizeof(CDS_Parameters) returns two different values
 * depending on which compiler is being used. 2162 for the firmware, 2164
 * for the host driver.
 *
 * The fix here is a kludge, until this difference is understood. An extra-bad
 * kludge, since if the internal structs in bhn_cds_parameters change size
 * they will need to be broken down and imitated as well. Currently due to
 * the cds padding they are all 4 byte aligned though, so it's good for now.
 */
#ifdef BHN_LINUX_HOST_DRIVER
#define BHN_CDS_PARAMETERS_SZ	(sizeof(struct bhn_cds_parameters))
#else
#define BHN_CDS_PARAMETERS_SZ	(sizeof(CDS_Parameters))
#endif
#define GET_CDS_CMD_SZ		(BHN_CMD_SZ)
#define GET_CDS_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_CDS_PARAMETERS_SZ)

#ifdef BHN_LINUX_HOST_DRIVER
struct bhn_cds_parameters {
  CDS_Information CDS_Info;
  BHN_Information BHN_Info;
  Network_Information NetInfo;
  Boot_Information BootUP;
  SerialConfigMgr SCM;
  SystemMgr SysMgr;
  Port_Information PortInfo;
  BHN_PortSettings Pset[MAX_PORTS];
  word end_of_params;
} __attribute__((packed));
#endif

/* GET_BHN_INFO */

#define BHN_INFO_CMD_SZ			(sizeof(struct bhn_info_cmd))
#define BHN_INFO_SZ			(sizeof(struct bhn_info))
#define GET_BHN_INFO_CMD_SZ		(BHN_CMD_SZ + BHN_INFO_CMD_SZ)
#define GET_BHN_INFO_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_INFO_SZ)

struct bhn_info_cmd {
  unsigned char bhn_mac[6];
  unsigned char bhn_hostname[HOST_NAME_SIZE];
} __attribute__((packed));

struct bhn_info {
  unsigned int bhn_ser_num;
  unsigned char bhn_ip[IPV4_SZ];
  unsigned char bhn_ip_v6[IPV6_SZ];
  unsigned char bhn_mac[6];
  unsigned char bhn_hostname[HOST_NAME_SIZE];
  unsigned char bhn_domainname[DOMAIN_NAME_SIZE];
  unsigned char cds_ver[VER_NUM_SIZE];
  unsigned char num_ports;
} __attribute__((packed));

/* GET_PORT_INFO */

#define BHN_PORTINFO_SZ			(sizeof(struct bhn_portinfo))
#define GET_PORT_INFO_CMD_SZ		(BHN_CMD_SZ)
#define GET_PORT_INFO_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_PORTINFO_SZ)

struct bhn_portinfo {
  unsigned char ip[IPV4_SZ];
  unsigned char ipv6[IPV6_SZ];
  unsigned char mac[6];
  unsigned char open;
  unsigned int rx_bytes;
  unsigned int tx_bytes;
  unsigned int frames;
  unsigned int hard_oruns;
  unsigned int soft_oruns;
  unsigned int paritys;
} __attribute__((packed));

/* SET_CONNECTION */

#define BHN_CSET_SZ			(sizeof(struct bhn_cset))
#define SET_CONNECTION_CMD_SZ		(BHN_CMD_SZ + BHN_CSET_SZ)
#define SET_CONNECTION_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

struct bhn_cset {
  unsigned char ip[IPV4_SZ];
  unsigned char ipv6[IPV6_SZ];
  unsigned char mac[6];
  unsigned char mode;
  unsigned char tportnum;
  unsigned char master;
} __attribute__((packed));

/* FATAL_ERROR */

#define FATAL_ERROR_CMD_SZ	(BHN_CMD_SZ)

/* PORT_RESET */

#define PORT_RESET_CMD_SZ	(BHN_CMD_SZ)

/* RESET_BHN */

#define RESET_BHN_CMD_SZ	(BHN_CMD_SZ)

/* SEND_IMMEDIATE */

#define BHN_SENDI_SZ		(sizeof(struct bhn_sendi))
#define SEND_IMMEDIATE_CMD_SZ	(BHN_CMD_SZ + BHN_SENDI_SZ)

struct bhn_sendi {
  unsigned char chr;
} __attribute__((packed));

/* FIFO_CONTROL */

#define BHN_FIFO_SZ			(sizeof(struct bhn_fifo))
#define FIFO_CONTROL_CMD_SZ		(BHN_CMD_SZ + BHN_FIFO_SZ)
#define FIFO_CONTROL_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_FIFO_SZ)

struct bhn_fifo {
  unsigned char rx;
  unsigned char tx;
} __attribute__((packed));

/* SET_EVENT / GET_EVENT */

#define BHN_EVENT_SZ		(sizeof(struct bhn_event))
#define SET_EVENT_CMD_SZ	(BHN_CMD_SZ + BHN_EVENT_SZ)
#define SET_EVENT_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)
#define GET_EVENT_CMD_SZ	(BHN_CMD_SZ)
#define GET_EVENT_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_EVENT_SZ)

#define EVENT_NONE	0x0000
#define EVENT_OE	0x0001
#define EVENT_PE	0x0002
#define EVENT_FE	0x0004
#define EVENT_BREAK	0x0008
#define EVENT_CTS	0x0010
#define EVENT_DSR	0x0020
#define EVENT_RING	0x0040
#define EVENT_DCD	0x0080
#define EVENT_RX_XON	0x0100
#define EVENT_RX_XOFF	0x0200
#define EVENT_TXFIN	0x0400
#define EVENT_ALL	0x07FF

struct bhn_event {
  unsigned short events;
} __attribute__((packed));

/* EVENT_OCCURRED */

#define BHN_EVENT_OCCURRED_SZ		(sizeof(struct bhn_event_occurred))
#define EVENT_OCCURRED_CMD_SZ		(BHN_CMD_SZ + BHN_EVENT_OCCURRED_SZ)
#define EVENT_OCCURRED_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

struct bhn_event_occurred {
  unsigned short events;
  unsigned char msr;
  int oe_count;
  int pe_count;
  int fe_count;
  int brk_count;
} __attribute__((packed));

/* GET_MSR */

#define BHN_MSR_SZ		(sizeof(struct bhn_msr))
#define GET_MSR_CMD_SZ		(BHN_CMD_SZ)
#define GET_MSR_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_MSR_SZ)

struct bhn_msr {
  unsigned char msr;
} __attribute__((packed));

/* SEND_XCHAR */

#define BHN_XCHAR_SZ		(sizeof(struct bhn_xchar))
#define SEND_XCHAR_CMD_SZ	(BHN_CMD_SZ + BHN_XCHAR_SZ)
#define SEND_XCHAR_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

#define XCHAR_XON	0x01
#define XCHAR_XOFF	0x02

struct bhn_xchar {
  unsigned char xchar;
} __attribute__((packed));

/* SET_RTS_DTR */

#define BHN_RTSANDDTR_SZ	(sizeof(struct bhn_rtsanddtr))
#define SET_RTS_DTR_CMD_SZ	(BHN_CMD_SZ + BHN_RTSANDDTR_SZ)
#define SET_RTS_DTR_CMD_DONE_SZ	(BHN_CMD_DONE_SZ)

struct bhn_rtsanddtr {
  unsigned char rts_onoff;
  unsigned char dtr_onoff;
} __attribute__((packed));

/* GET_PORTCAP */

#define BHN_PORTCAP_SZ		(sizeof(struct bhn_portcap))
#define GET_PORTCAP_CMD_SZ	(BHN_CMD_SZ)
#define GET_PORTCAP_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_PORTCAP_SZ)

struct bhn_portcap {
  unsigned char capability[MAX_PORTS];
} __attribute__((packed));

/* GET_MFGINFO */

#define BHN_MFGINFO_SZ		(sizeof(struct bhn_mfginfo))
#define GET_MFGINFO_CMD_SZ	(BHN_CMD_SZ)
#define GET_MFGINFO_CMD_DONE_SZ	(BHN_CMD_DONE_SZ + BHN_MFGINFO_SZ)

struct bhn_mfginfo {
  unsigned char info[FL_CDS_SIZE];
} __attribute__((packed));

/* Command done header */

#define BHN_CMD_DONE_SZ	(sizeof(struct bhn_cmd_done))

#define CMD_OK				0
#define BHN_ERROR_OPENED		1
#define BHN_ERROR_INVALID_PORT		2
#define BHN_ERROR_UNSPECIFIED		3
#define BHN_ERROR_INVALID_BAUD		4
#define BHN_ERROR_INVALID_STOP_BITS	5
#define BHN_ERROR_INVALID_PARITY	6
#define BHN_ERROR_INVALID_DATA_BITS	7

struct bhn_cmd_done {
  unsigned char cmd_done;
  unsigned char cmd_completed;
  unsigned char port;
  unsigned char status;
  unsigned short length;
} __attribute__((packed));

#ifndef MAX
#define MAX(a, b)	((a) > (b) ? (a) : (b))
#endif
#define MAX_CMD_DATA_SZ	(					\
				MAX(BHN_OPEN_SZ,		\
				MAX(BHN_PURGE_SZ,		\
				MAX(BHN_LSET_SZ,		\
				MAX(BHN_LIM_SZ,			\
				MAX(BHN_BREAK_SZ,		\
				MAX(BHN_SSBREAK_SZ,		\
				MAX(BHN_RTSDTR_SZ,		\
				MAX(BHN_INFO_CMD_SZ,		\
				MAX(BHN_CSET_SZ,		\
				MAX(BHN_SENDI_SZ,		\
				MAX(BHN_FIFO_SZ,		\
				MAX(BHN_EVENT_SZ,		\
				MAX(BHN_EVENT_OCCURRED_SZ,	\
				MAX(BHN_XCHAR_SZ,		\
				MAX(BHN_RTSANDDTR_SZ,		\
				MAX(0, 0)			\
				)))))))))))))))			\
			)
#define MAX_CMD_DONE_DATA_SZ	(					\
					MAX(BHN_LSET_SZ,		\
					MAX(BHN_LIM_SZ,			\
					MAX(BHN_LSTATE_SZ,		\
					MAX(BHN_CDS_PARAMETERS_SZ,	\
					MAX(BHN_INFO_SZ,		\
					MAX(BHN_PORTINFO_SZ,		\
					MAX(BHN_FIFO_SZ,		\
					MAX(BHN_EVENT_SZ,		\
					MAX(BHN_MSR_SZ,			\
					MAX(BHN_PORTCAP_SZ,		\
					MAX(BHN_MFGINFO_SZ,		\
					MAX(0, 0)			\
					)))))))))))			\
				)
#define MAX_CMD_SZ		(BHN_CMD_SZ + MAX_CMD_DATA_SZ)
#define MAX_CMD_DONE_SZ		(BHN_CMD_DONE_SZ + MAX_CMD_DONE_DATA_SZ)
#define MAX_NET_CMD_SZ		(MAX(MAX_CMD_SZ, MAX_CMD_DONE_SZ))

struct bhn_prot_cmd {
  union {
    unsigned char _pcbuf[MAX_CMD_SZ];
    struct {
      struct bhn_cmd _pccmd;
      union {
        struct bhn_open _pcopen;
        struct bhn_purge _pcpurge;
        struct bhn_lset _pclset;
        struct bhn_lim _pclim;
        struct bhn_break _pcbreak;
        struct bhn_ssbreak _pcssbreak;
        struct bhn_rtsdtr _pcrtsdtr;
        struct bhn_info_cmd _pcinfo_cmd;
        struct bhn_cset _pccset;
        struct bhn_sendi _pcsendi;
        struct bhn_fifo _pcfifo;
        struct bhn_event _pcevent;
        struct bhn_event_occurred _pcevent_occurred;
        struct bhn_xchar _pcxchar;
        struct bhn_rtsanddtr _pcrtsanddtr;
      } __attribute__((packed)) u3;
    } __attribute__((packed)) u2;
  } __attribute__((packed)) u1;
} __attribute__((packed));

#define pc_buf			u1._pcbuf
#define pc_cmd			u1.u2._pccmd
#define pc_open			u1.u2.u3._pcopen
#define pc_purge		u1.u2.u3._pcpurge
#define pc_lset			u1.u2.u3._pclset
#define pc_lim			u1.u2.u3._pclim
#define pc_break		u1.u2.u3._pcbreak
#define pc_ssbreak		u1.u2.u3._pcssbreak
#define pc_rtsdtr		u1.u2.u3._pcrtsdtr
#define pc_info_cmd		u1.u2.u3._pcinfo_cmd
#define pc_cset			u1.u2.u3._pccset
#define pc_sendi		u1.u2.u3._pcsendi
#define pc_fifo			u1.u2.u3._pcfifo
#define pc_event		u1.u2.u3._pcevent
#define pc_event_occurred	u1.u2.u3._pcevent_occurred
#define pc_xchar		u1.u2.u3._pcxchar
#define pc_rtsanddtr		u1.u2.u3._pcrtsanddtr

struct bhn_prot_cmd_done {
  union {
    unsigned char _prbuf[MAX_CMD_DONE_SZ];
    struct {
      struct bhn_cmd_done _prcmd_done;
      union {
        struct bhn_lset _prlset;
        struct bhn_lim _prlim;
        struct bhn_lstate _prlstate;
#ifdef BHN_LINUX_HOST_DRIVER
	struct bhn_cds_parameters _prcdsp;
#else
        CDS_Parameters _prcdsp;
#endif
        struct bhn_info _prbinfo;
        struct bhn_portinfo _prportinfo;
        struct bhn_fifo _prfifo;
        struct bhn_event _prevent;
        struct bhn_msr _prmsr;
        struct bhn_portcap _prportcap;
        struct bhn_mfginfo _prmfginfo;
      } __attribute__((packed)) u3;
    } __attribute__((packed)) u2;
  } __attribute__((packed)) u1;
} __attribute__((packed));

#define pr_buf		u1._prbuf
#define pr_cmd_done	u1.u2._prcmd_done
#define pr_lset		u1.u2.u3._prlset
#define pr_lim		u1.u2.u3._prlim
#define pr_lstate	u1.u2.u3._prlstate
#define pr_cdsp		u1.u2.u3._prcdsp
#define pr_binfo	u1.u2.u3._prbinfo
#define pr_portinfo	u1.u2.u3._prportinfo
#define pr_fifo		u1.u2.u3._prfifo
#define pr_event	u1.u2.u3._prevent
#define pr_msr		u1.u2.u3._prmsr
#define pr_portcap	u1.u2.u3._prportcap
#define pr_mfginfo	u1.u2.u3._prmfginfo

/* Reset the packing alignment to the default (8) */
#ifdef _WINDOWS
  #pragma pack()
#endif

#endif
