/*******************************************************************************************
**	CDS.h
**	Copyright 2003,2004 Connect Tech. Inc.
********************************************************************************************/

/*
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	STOP!! DON'T MODIFY THIS FILE
	The contents of this file are use by MANY applications.

	This file is maintained as part of the CDS/Boot/Loader code base, and should
	be modified ONLY in that build process.

	So far the following applications use this file:
		Boot/Loader
		uClinux
		TCM
		SCM

	When modified (as indicated above), it should be distributed to all the applications
	which use it.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

#ifndef _CDS_H
	#define _CDS_H

	// IMPORTANT NOTE: These version numbers are placed into this file by
	//	the build process (Makefile) of the Boot/Loader,
	//	and SHOULD NOT BE MODIFIED DIRECTLY
	#define CDS_VERSION "V11"

	#define DOMAIN_NAME_SIZE	128
	#define HOST_NAME_SIZE		128
	#define FILENAME_SIZE		128
	#define PW_SIZE				16
	#define KW_SIZE				16
	#define MAX_PORTS			16
	#define VER_NUM_SIZE		32

	#define SCM_DEFAULT_PASSWORD		"password"
	#define ROOT_DEFAULT_PASSWORD		"root"

	typedef unsigned char byte;
	typedef unsigned short word;
	typedef unsigned long dword;

	#define IPV4_SZ		4
	#define IPV6_SZ		16
	typedef struct {
		byte v6[IPV6_SZ];		// Future IPv6 addressing (this member MUST be first,
								//		to allow initialization constants to compiled correctly)
		byte v4[IPV4_SZ];		// IPv4 addressing
	} UIP_ADDR;

	//=============================================================================================
	// Serial Port settings
	typedef struct {
		dword baud;					//	Baud rate;
		dword xoff_lim;
		dword xon_lim;
		byte use_xlim;				//	Enable/Disable Xoff/Xon limits
		byte mode;					//	Line operational mode (see file BHN_485.h)
		byte bits;					//	Data bits; 5,6,7 or 8
		byte parity;				//	Parity;
									//		'n,e,o,0 or 1'
									//			'n'=none,	'e'=even, 'o'=odd, '0'=force 0, '1'=force 1
		byte stop;					//	Stop bits;
									//		0=1, 1=2 (unless data bits=5, where 1 is actually 1.5)
		byte sflow;					//	Software (in-band) flow control modes;
									//		'n,r,t,b'
									//			'n'=none
									//			'r'=receive (Xoff/On transmitted when receiver fills/empties)
									//			't'=transmit (Xoff/On received will stop/start TX)
									//			'b'=both (receive and transmit)
		byte xoff;					//	Xoff value
		byte xon;					//	Xon value
		byte hflow;					//	Hardware (outof-band) floe control modes;
									//		CTS (bit3) (CTS Off/On will cause TX Off/On)
									//		DSR (bit4) (DSR Off/On will cause TX Off/On)
									//		RTS (bit7) (RTS Off/On when receiver fills/empties)
									//		DTR (bit1) (DTR Off/On when receiver fills/empties)
									//		RTS_TOGGLE (bit0) (RTS is ON during TX)
		byte lloop;					//	Local Loopback; 0=Off, !=0 is On
		byte special_char_mode;		//	Enable/Disable ErrorChar, BreakChar, EventChar mode
									//		bit0=ErrorChar, bit1=BreakChar, bit2=EventChar
									//		bit4=turn on NULL stripping
		byte error_char;			//	Char to put into stream when POF errors occur
		byte break_char;			//	Char to put into stream when RX break occurs
		byte event_char;			//	Char to put into stream when Event occurs (not sure what this means)
		byte escape_char;			//	Char which preceeds other error chars (see Windows Driver)
		byte future_use[37];
	} BHN_PortSettings;


	//=============================================================================================
	// Version Information Strings
	typedef struct {
		char BootLdrSCM_package[VER_NUM_SIZE];		// Version Numbers (intended to hold strings)
		char uClinux_package[VER_NUM_SIZE];
	} Version_Numbers;


	//=============================================================================================
	// Basic BHN unit Information
	typedef struct {
		dword SerialNumber;				// Serial number in binary form
		Version_Numbers Ver;
		byte future_use[28];
	} BHN_Information;


	//=============================================================================================
	// !!! NOTE: This version number is specified in the 'Makefile', and is built into the output
	//			files by the 'make' process
	//	!!!	THIS VERSION NUMBER MUST!!!! BE CHANGED IN THE MAKEFILE WHENEVER ANY CHANGES
	//		ARE MADE TO ANY OF THE MEMBERS OF THE CDS AREA !!!
	typedef struct {
		char Version[VER_NUM_SIZE];				// Version number 'string'
		byte future_use[64];
	} CDS_Information;


	//=============================================================================================
	// Network Settings
	typedef struct {
		byte MAC_Address[6];				// FEC MAC/Link layer address
		UIP_ADDR CardIP;					// IP address of this card
		UIP_ADDR ServerIP;					// Boot Server IP address
		UIP_ADDR GateIP;					// Gateway IP address
		UIP_ADDR SubnetMask;				// Subnet mask
		UIP_ADDR BroadcastIP;				// Broadcast IP
		UIP_ADDR DNS_IP;					// DNS IP address
		byte DomainName[DOMAIN_NAME_SIZE];	// Domain name
		byte HostName[HOST_NAME_SIZE];		// Host Name
		word TCP_Port;						// Starting (base) TCP port number
		word DHCP_Enab;						// Use DHCP (the use of a 'word' retains 'word' alignment)
		byte future_use[78];
	} Network_Information;


	//=============================================================================================
	// Boot-up Settings
	typedef struct {
		byte BootMode;			//	Bits that control boot modes
								//		Bit-0; Enable FLASH boot
								//		Bit-1; Enable Bootp service
								//		Bit-2; Enable TFTP download
								//	Some combinations make no practical sense (like having Bootp enabled without TFTP).
								//	Some meaningful values
								//		1	Boot from FLASH only
								//		7	Attempt FLASH boot, if that fails, try to find Bootp server
								//				(to obtain IP addresses and boot filename),	and if that fails,
								//				try a TFTP using the CDS values.
								//		5	Attempt FLASH boot, then TFTP (using CDS values)
								//		6	Bootp then TFTP
								//		4	TFTP only
								//		0	None, this effectively causes the SCM to be entered always,
								//				and the ScanTimeout value to be ignored.
		byte BootFile[FILENAME_SIZE];		// TFTP file name (might contain a path too) (filled in by bootp "reply")
		byte future_use[31];
	} Boot_Information;

	// Boot Sequences (values for above 'BootMode')
	#define BOOT_F		1		// FLASH only (out-of-the-box default)
	#define BOOT_B		2		// Not a valid setting, but useful for testing this bit
	#define BOOT_T		4		// TFTP only
	#define BOOT_FT		5		// FLASH then TFTP
	#define BOOT_BT		6		// Bootp then TFTP
	#define BOOT_FBT	7		// FLASH then BootP then TFTP
	#define BOOT_NONE	0		// None (SCM is entered always)

	// Bootp utilization of various parameters
	//	CardIP			filled in by bootp "reply"
	//	ServerIP		filled in by bootp "reply"
	//	GateIP			used by bootp "request"
	//	BroadcastIP		used by bootp "request"
	//	BootFile		filled in by bootp "reply"


	//=============================================================================================
	// SCM Operation parameters
	typedef struct {
		word Password_changes;		// Count of the number of times the PassWord has changed (set to zero when BHNet is new)
									//	(this is mainly used to indicate if the user has ever changed the password since the
									//	unit was received).
		byte ScanPort;				// Ports to scan to enter SCM (1->16=off chip, 255=scan all serial ports)
		byte ScanTimeout;			// Time to wait for entry into the SCM (seconds)
		byte ScanPrompt;			// Issue a prompt for the keyword (when this value is non-zero)
		byte PassWord[PW_SIZE];		//	Password required to enter either the SCM or TCM or WCM Configuration Utilities
									//		On the SCM, the oportunity to enter the utility is only presented at the
									//		beginning of the bootup, for a duration and port# as defined in
									//		this structure
		byte KeyWord[KW_SIZE];		// Keyword required to increase permission level
		byte LoaderFile[FILENAME_SIZE];	//	Default file name for the downloaded "Loader/SCM"
		byte future_use[43];
	} SerialConfigMgr;


	//=============================================================================================
	// Telnet/Web Configuration Manager settings
	#define DEFAULT_SYS_TIMEOUT		300
	typedef struct {
		word Password_changes;		// Count of the number of times this password is changed
		word ConfigTimeout;			// TCM/WCM session inactivity timeout (seconds)
		byte PassWord[PW_SIZE];		// Password which is used to get into the OS from a telnet session
		byte future_use[44];
	} SystemMgr;


	//=============================================================================================
	// Serial Port Informatiomn
	typedef struct {
		byte NumPorts;					// Number of serial ports (discovered by "loader")
		byte UartType[MAX_PORTS];		// Type of Uarts; (See function: GetUartType for values) 
		byte future_use[31];
	} Port_Information;
		

	//=============================================================================================
	// All active CDS parameters

	typedef struct {
		CDS_Information CDS_Info;				// see above
		BHN_Information BHN_Info;				// see above
		Network_Information NetInfo;			// see above
		Boot_Information BootUP;				// see above
		SerialConfigMgr SCM;					// see above
		SystemMgr SysMgr;						// see above
		Port_Information PortInfo;				// see above
		BHN_PortSettings Pset[MAX_PORTS];		// Off-chip serial ports
		word end_of_params;
	} CDS_Parameters;


	//=============================================================================================
	// The entire CDS space including the CRC
	#define CDS_IN_RAM		0x400		// location of the CDS space in RAM


	typedef struct {
		// Actual active CDS values (see above data types)
		CDS_Parameters CDS_P;

		// The following padding is used to position the CRC at the end of the CDS space,
		// this is required, as part of the strategy, for CDS space integrity preservation.
		unsigned char padding[FL_CDS_SIZE - sizeof(CDS_Parameters) - sizeof(unsigned short)];

		// CRC CCITT-16 (see code in file CDS_support.c)
		unsigned short CRC;
	} CDS_Space;


	//=============================================================================================
	// Run-Time parameters
	#define RTP_IN_RAM		(CDS_IN_RAM + FL_CDS_SIZE)		// RAM address of the following structure
	typedef struct {
		unsigned long lock;		// used in uClinux to create exclusive use of certain resources
		word cds_sector;		// holds the FLASH sector address of the CDS which was used to create the RAM copy
								//	(this is required for any subsequent saving of the CDS back to the FLASH memory)
		dword loader_sector;	// holds the FLASH sector address of the Loader which was extracted into RAM for execution.
								//	(this is required for any subsequent saving of the Loader back to the FLASH memory)
	} RunTimeParam;
#endif
