/*
 * ksocket.h
 *
 * The kernel-friendly socket layer.
 *
 * For the most part these calls work exactly as they would in user space;
 * there are some exceptions:
 * Where a call would normally take a file descriptor it now takes a socket.
 * Since calls can't pick up O_NONBLOCK from the fd, MSG_NOWAIT must be
 * explicitly handled by the caller in the flags argument.
 * The return types are int instead of long, since only errnos are returned.
 * ksys_socket has an extra parameter to return the newly created socket.
 * ksys_accept and ksys_connect have an extra parameter to pass explicit fd
 * flags. Only O_NONBLOCK seems to be important though.
 *
 * Version:	0.10
 * Date:	Sep 27 2004
 * Author:	Stuart MacDonald <stuartm@connecttech.com>
 *
 */
#ifndef _KSOCKET_H
#define _KSOCKET_H

int ksys_socket(int family, int type, int protocol, struct socket **sock);
int ksys_socketpair(int family, int type, int protocol,
						struct socket *sockvec[2]);
int ksys_bind(struct socket *sock, struct sockaddr *addr, int addrlen);
int ksys_listen(struct socket *sock, int backlog);
int ksys_accept(struct socket *sock, struct sockaddr *peer_sockaddr,
						int *peer_addrlen, int flags);
int ksys_connect(struct socket *sock, struct sockaddr *addr, int addrlen,
								int flags);
int ksys_getsockname(struct socket *sock, struct sockaddr *sockaddr,
							int *sockaddr_len);
int ksys_getpeername(struct socket *sock, struct sockaddr *sockaddr,
							int *sockaddr_len);
int ksys_sendto(struct socket *sock, void *buff, size_t len, unsigned flags,
					struct sockaddr *addr, int addr_len);
int ksys_send(struct socket *sock, void *buff, size_t len, unsigned flags);
int ksys_recvfrom(struct socket *sock, void *ubuf, size_t size, unsigned flags,
					struct sockaddr *addr, int *addr_len);
int ksys_recv(struct socket *sock, void *ubuf, size_t size, unsigned flags);
int ksys_setsockopt(struct socket *sock, int level, int optname, char *optval,
								int optlen);
int ksys_getsockopt(struct socket *sock, int level, int optname, char *optval,
								int *optlen);
int ksys_shutdown(struct socket *sock, int how);
int ksys_sendmsg(struct socket *sock, struct msghdr *msg, unsigned int flags);
int ksys_recvmsg(struct socket *sock, struct msghdr *msg, unsigned int flags);

#endif
