#!/bin/sh

#
# Copyright 2005 Connect Tech Inc.
# http://www.connecttech.com
#
# This is the install script for the Blue Heat/Net Linux Host Driver.
# It will unpack the source, compile the module, load it, set up devices,
# install boot scripts.
#

declare -r SCRIPT_VERSION="1.02"
declare -r SCRIPT_DATE="Mar 7 2006"
declare -r CONFIG_VERSION="1.00"

#
# History
#
# 1.02 Mar 7 2006 cjd
# -Cleanup.
#
# 1.01 Mar 2 2006 cjd
# -Separated script version from config file version.
#
# 1.00 Feb 28 2006 cjd
# -Created new script with the following features:
#   -2.4 and 2.6 support.
#   -Uninstall capabilities.
#   -Log file generation.
#   -Create/use config file.
# -Fixed require() so it can handle names containing "-" or ".".
#
# 0.17 Jan 19 2006 glh
# -Fixed a few spelling mistakes.
# -Boot-time scripts now check for "bhn" and "ksocket_nosomaxconn"
#  modules before trying to remove them.
# -During the do_install the script now checks to see if the modules
#  are loaded before modprobing them again.
#
# 0.16 Jan 19 2006 cjd
# -Fixed up some UI experience/flow issues.
#
# 0.15 Jan 18 2006 cjd
# -Added DISTRO display to confirmation.
# -Condensed history information.
#
# 0.14 Jan 18 2006 cjd
# -Cleanup.
# -Fixed support for Debian and modules.conf.
#
# 0.13 Jan 18 2006 cjd
# -Added removal of ksocket_nosomaxconn to init script's stop.
#
# 0.12 Jan 18 2006 cjd
# -Fixed loading of modules despite answering "no" to the "kernel running"
#  question.
#
# 0.11 Jan 17 2006 cjd
# -Fixed configuration file prompts.
# -General cleanup.
# -Fixed INIT_SCRIPT_DIR.
# -Renamed INIT_SCRIPT_DIR to SCRIPT_DIR.
#
# 0.10 Jan 16 2006 glh
# -Added question about current kernel being the same as the build target.
# -Fixed relative path problem by adding make_absolute_path function.
# -Reorganized autodiscovery questions and explanation.
# -Fixed kernel build target prompting.
# -Added INIT_SCRIPT_DIR.
# -Removed useless help for command line options and options themselves.
# -Added Distribution detection.
# -Added Debian boot script.
# -Fixed Proceed with these settings prompting (needs more work).
# -Added clear-mappings keyword usage.
# -Discovered 'require' cannot handle commands with - or . ... there are more.
#
# 0.09 Jan 12 2006 glh
# -Major additions, user is asked quite a few questions about install.
# -Install now takes you through to a loaded module with /dev nodes created.
# -Removing now-defunct command line argument processing.  We should not need
#  this in the future.  Also, the help message says to contact Tech Support
#  if you have problems.
#
# 0.08 Dec 22 2005 glh
# -Added the DEFAULT_BHN_LHD_DIR and setup for this var in the parent Makefile.
# -Fixed default values for driver installation:
#  -Overwrite by default
#  -Start driver by default
#  -Proceed with these settings by default
# -Fixed the KERNEL_DIR Makefile AWK script.
#
# 0.07 Sep 26 2005 stuartm
# -Driver tarball now supplied by the Makefile during packaging. Necessary
#  to support separate 2.4 and 2.6 drivers.
#
# 0.06 Mar 10 2005 stuartm
# -Added better handling for the case of overwriting an existing install.
#
# 0.05 Feb 18 2005 stuartm
# -First draft.
#

#
# Global variables
#

# General
declare THIS="$0"
declare START_DIR=""
declare BASE_DIR=""
declare SIG_MASK="0 1 2 3 9 11 13 15"
declare -i SIG_EXIT=1

# KERNEL_VERSION defines which kernel the driver is for
# KERNEL_VERSION=
#   0 - 2.4 based kernels
#   1 - 2.6 based kernels
declare -i KERNEL_VERSION=0

declare BHN_LHD_SOURCE="BHN-LHD-24-2.16.tar.gz"
declare BHN_LHD_DIR="BHN-LHD-24-2.16"
declare SCRIPT_DIR="scripts"
declare DISTRO_ID=""
declare DISTRO_NAME=""
declare CONFIG_FILE_VERSION=""

# Constants
declare -r KERNEL_BUILDS="/lib/modules"
declare -r UNINSTALL_CONFFILE="uninst.conf"

declare -r -i DEFAULT_CL_LOG=0
declare -r DEFAULT_CL_LOGFILE="install.log"
declare -r -i DEFAULT_CL_GCONF=0
declare -r DEFAULT_CL_GCONFFILE="install.conf"
declare -r -i DEFAULT_CL_UCONF=0
declare -r DEFAULT_CL_UCONFFILE="install.conf"
declare -r -i DEFAULT_CL_FORCE=0

declare -r DEFAULT_INST_DIR="/usr/cti"
declare -r -i DEFAULT_INST_KERNEL_RUNNING=1
declare -r -i DEFAULT_INST_LOAD_DRIVER=1
declare -r -i DEFAULT_INST_AUTOD=1
declare -r -i DEFAULT_INST_CONFIG_DRIVER=1
declare -r -i DEFAULT_INST_INIT_SCRIPT=1
declare -r DEFAULT_INST_AUTOD_CONFIG_FILE="/etc/bhn.conf"
declare -r DEFAULT_INST_CONFIG_FILE="/etc/bhn.conf"
declare -r DEFAULT_INST_INIT_SCRIPT_CONFIG_FILE="/etc/bhn.conf"

declare -r -i LOG_APPEND=0
declare -r -i LOG_NO_APPEND=1
declare -r -i LOG_NORMAL=0
declare -r -i LOG_WARN=1
declare -r -i LOG_ERR=2

# Command Line
declare -i CL_LOG=$DEFAULT_CL_LOG
declare CL_LOGFILE="$DEFAULT_CL_LOGFILE"
declare -i CL_GCONF=$DEFAULT_CL_GCONF
declare CL_GCONFFILE="$DEFAULT_CL_GCONFFILE"
declare -i CL_UCONF=$DEFAULT_CL_UCONF
declare CL_UCONFFILE="$DEFAULT_CL_UCONFFILE"
declare -i CL_FORCE=$DEFAULT_CL_FORCE

# Install
declare INST_DIR="$DEFAULT_INST_DIR"
declare INST_KERNEL_BUILD=""
declare -i INST_KERNEL_RUNNING=$DEFAULT_INST_KERNEL_RUNNING
declare -i INST_LOAD_DRIVER=$DEFAULT_INST_LOAD_DRIVER
declare -i INST_AUTOD=$DEFAULT_INST_AUTOD
declare -i INST_CONFIG_DRIVER=$DEFAULT_INST_CONFIG_DRIVER
declare -i INST_INIT_SCRIPT=$DEFAULT_INST_INIT_SCRIPT
declare INST_AUTOD_CONFIG_FILE="$DEFAULT_INST_AUTOD_CONFIG_FILE"
declare INST_CONFIG_FILE="$DEFAULT_INST_CONFIG_FILE"
declare INST_INIT_SCRIPT_CONFIG_FILE="$DEFAULT_INST_INIT_SCRIPT_CONFIG_FILE"


#
# Functions
#

#
# error - prints an error
#
# $1 - error message
#
error () {
  echo
  echo "$THIS: Error: $1"
}

#
# die - prints an error and exits
#
# $1 - exit code
# $2 - error message
#
die () {
  error "$2"
  SIG_EXIT=0
  exit $1
}

#
# require - checks to see if the util/file is available, and sets UTIL if
#           util is available
#
# $1 - "util" | "file"
# $2 - util or file name
#
require () {
  local util up_util

  if [ "$1" = "util" ]; then
    util=`which $2`
    if [ $? -ne 0 ] || [ "$util" = "" ]; then
      die 2 "Required utility '$2' not found"
    fi
    # awk in next line is special case. Can't use $AWK since $AWK hasn't
    # been set yet. But, as long as awk is the first util to be required,
    # the above check ensures it does exist at this point.
    up_util=`echo $2 | awk '{gsub("-","",$0); gsub("\\\.","",$0); print toupper($0)}'`
    eval "$up_util=$util"
  elif [ "$1" = "file" ]; then
    if [ ! -f "$2" ]; then
      die 16 "Required file '$2' not found"
    fi
  else
    die 1 "Bad argument '$1' to internal function require()"
  fi
}

#
# make_abs_path - Make an absolute path based on a
#                 directory
#
# $1 - original path
# $2 - base path
#
# output: absolute path
#
make_abs_path() {
  local orig_path="$1"
  local base_path="$2"

  case "$orig_path" in
    /*)
      echo "$orig_path"
      ;;
    *)
      case "$base_path" in
        */)
          echo "$base_path""$orig_path"
          ;;
        *)
          echo "$base_path/$orig_path"
          ;;
      esac
      ;;
  esac
}

#
# check_requirements - make sure required utils and files are available
#
check_requirements() {
  # awk must always be required first, since require() uses it
  require "util" "awk"
  require "util" "uname"
  require "util" "id"
  require "util" "date"
  require "util" "sed"
  require "util" "basename"
  require "util" "dirname"
  require "util" "ls"
  require "util" "mkdir"
  require "util" "gunzip"
  require "util" "tar"
  require "util" "mv"
  require "util" "make"
  require "util" "lsmod"
  require "util" "grep"
  require "util" "cut"
  require "util" "rmmod"
  require "util" "modprobe"
  require "util" "cat"
  require "util" "rm"
  require "util" "chmod"
}

#
# usage - display usage information
#
usage() {
  echo "$THIS - Blue Heat/Net Installation Script"
  echo
  echo "Command line arguments:"
  echo "  $THIS [-h]"
  echo "  $THIS [-v]"
  echo "  $THIS [-l<log_file>] [-g<conf_file>] [-c<conf_file> [-f]]"
  echo
  echo "  -c  use config file <conf_file> (default=$DEFAULT_CL_UCONFFILE)"
  echo "  -f  force config file (only takes effect when combined with -c)"
  echo "  -g  generate config file <conf_file> (default=$DEFAULT_CL_GCONFFILE)"
  echo "  -h  display help information and exit"
  echo "  -l  log operations to <log_file> (default=$DEFAULT_CL_LOGFILE)"
  echo "  -v  display install script version and exit"
  echo
  echo "This script is the install script for the Blue Heat/Net driver."
  echo "It should step you through the install and set up of a Blue Heat/Net."
  echo "Should you have trouble running this script, please contact Connect Tech"
  echo "technical support at support@connecttech.com"
}

#
# parse_command_line - parse command line options
#
# $1 - command line arguments
#
parse_command_line() {
  while [ $# != 0 ]; do
    case "$1" in
      -c*)
        CL_UCONF=1
        CL_UCONFFILE="`echo $1 | $SED 's/-c//'`"
        if [ "$CL_UCONFFILE" == "" ]; then
          CL_UCONFFILE="$DEFAULT_CL_UCONFFILE"
        fi
        CL_UCONFFILE="`make_abs_path "$CL_UCONFFILE" "$START_DIR"`"
        ;;
      -f)
        CL_FORCE=1
        ;;
      -g*)
        CL_GCONF=1
        CL_GCONFFILE="`echo $1 | $SED 's/-g//'`"
        if [ "$CL_GCONFFILE" == "" ]; then
          CL_GCONFFILE="$DEFAULT_CL_GCONFFILE"
        fi
        CL_GCONFFILE="`make_abs_path "$CL_GCONFFILE" "$START_DIR"`"
        ;;
      -h)
        usage
        exit 0
        ;;
      -l*)
        CL_LOG=1
        CL_LOGFILE="`echo $1 | $SED 's/-l//'`"
        if [ "$CL_LOGFILE" == "" ]; then
          CL_LOGFILE="$DEFAULT_CL_LOGFILE"
        fi
        CL_LOGFILE="`make_abs_path "$CL_LOGFILE" "$START_DIR"`"
        ;;
      -v)
        echo "$THIS version $SCRIPT_VERSION ($SCRIPT_DATE); config version $CONFIG_VERSION"
        exit 0
        ;;
      *)
        error "Unrecognised option '$1'"
        die 4 "'$THIS -h' for help"
        ;;
    esac
    shift
  done
}

#
# check_linux - verifies current OS is Linux
#
# exits script if current OS is not Linux
#
check_linux() {
  local os="`$UNAME -s`"

  if [ "$os" != "Linux" ]; then
    die 12 "You are not running Linux!"
  fi
}

#
# check_root - verifies current user is root
#
# exits script if current user is not root
#
check_root() {
  local uid

  uid=`$ID -u`

  if [ $uid -ne 0 ]; then
    die 5 "Must be root"
  fi
}

#
# log - logs output to a file
#
# $1 - text to log
# $2 - 0 = append to file
#      1 = do not append to file
# $3 - severity  0 = normal
#                1 = warning
#                2 = error
#
log() {
  local log_text="$1"
  local -i no_append=$2
  local -i severity=$3
  local stamp="`$DATE +%Y%m%d-%T\#\ `"

  if [ $CL_LOG -eq 1 ]; then
    if [ $no_append -eq 0 ]; then
      echo -n "$stamp" >>$CL_LOGFILE
    else
      echo -n "$stamp" >$CL_LOGFILE
    fi
    [ $severity -eq 1 ] && echo -n "{Warning} " >>$CL_LOGFILE
    [ $severity -eq 2 ] && echo -n "{Error} " >>$CL_LOGFILE
    echo "$log_text" >>$CL_LOGFILE
  fi
}

#
# do_exit - handle cleanup
#
do_exit() {
  if [ $SIG_EXIT -eq 1 ]; then
    echo
    log "Signal caught." $LOG_APPEND
  fi

  if [ -f "$INST_DIR/$BHN_LHD_DIR/Makefile.$$" ]; then
    $RM -f $INST_DIR/$BHN_LHD_DIR/Makefile.$$
  fi

  cd $START_DIR
  trap - $SIG_MASK
  log "Finished Blue Heat/Net install script." $LOG_APPEND
  exit
}

#
# gen_conf - generate a configuration file
#
gen_conf() {

  $CAT << END_OF_FILE > $CL_GCONFFILE

CONFIG_FILE_VERSION="$CONFIG_VERSION"
DISTRO_ID="$DISTRO_ID"

INST_DIR="$INST_DIR"
INST_KERNEL_BUILD="$INST_KERNEL_BUILD"
INST_KERNEL_RUNNING=$INST_KERNEL_RUNNING
INST_LOAD_DRIVER=$INST_LOAD_DRIVER
INST_AUTOD=$INST_AUTOD
INST_CONFIG_DRIVER=$INST_CONFIG_DRIVER
INST_INIT_SCRIPT=$INST_INIT_SCRIPT
INST_AUTOD_CONFIG_FILE="$INST_AUTOD_CONFIG_FILE"
INST_CONFIG_FILE="$INST_CONFIG_FILE"
INST_INIT_SCRIPT_CONFIG_FILE="$INST_INIT_SCRIPT_CONFIG_FILE"

END_OF_FILE

  eval "$DISTRO_ID""_gen_conf"
}

#
# gen_uninst - generate uninstall script
#
gen_uninst() {

  $AWK " {
if (/^declare -r SCRIPT_VERSION=/) print \"declare -r SCRIPT_VERSION=\\\"$SCRIPT_VERSION\\\"\";
else if (/^declare -r SCRIPT_DATE=/) print \"declare -r SCRIPT_DATE=\\\"$SCRIPT_DATE\\\"\";
else if (/^declare -r CONFIG_VERSION=/) print \"declare -r CONFIG_VERSION=\\\"$CONFIG_VERSION\\\"\";
else if (/^declare -i KERNEL_VERSION=/) print \"declare -i KERNEL_VERSION=$KERNEL_VERSION\";
else if (/^declare BHN_LHD_DIR=/) print \"declare BHN_LHD_DIR=\\\"$BHN_LHD_DIR\\\"\";
else if (/^declare SCRIPT_DIR=/) print \"declare SCRIPT_DIR=\\\"$SCRIPT_DIR\\\"\";
else if (/^declare -r DEFAULT_CL_UCONFFILE=/) print \"declare -r DEFAULT_CL_UCONFFILE=\\\"$UNINSTALL_CONFFILE\\\"\";
else print;
} " < $BASE_DIR/uninst.sh.template > $INST_DIR/$BHN_LHD_DIR/uninst.sh

  $CHMOD a+x $INST_DIR/$BHN_LHD_DIR/uninst.sh

  $CAT << END_OF_FILE > $INST_DIR/$BHN_LHD_DIR/$UNINSTALL_CONFFILE

CONFIG_FILE_VERSION="$CONFIG_VERSION"
DISTRO_ID="$DISTRO_ID"

INST_DIR="$INST_DIR"
INST_KERNEL_BUILD="$INST_KERNEL_BUILD"
INST_INIT_SCRIPT=$INST_INIT_SCRIPT

END_OF_FILE

  eval "$DISTRO_ID""_gen_uninst_conf"
}

#
# _commit_install_install_source
#
_commit_install_install_source() {

  if [ -d "$INST_DIR/$BHN_LHD_DIR" ]; then
    log "Previous installation exists at <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND
    $RM -rf $INST_DIR/$BHN_LHD_DIR
    if [ $? -ne 0 ]; then
      log "Failed to remove previous installation from <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND $LOG_ERR
      die 18 "Unable to remove old installation"
    fi
    log "Removed previous installation from <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND
  fi

  $MKDIR -p $INST_DIR
  if [ $? -ne 0 ]; then
    log "Failed to create directory <$INST_DIR>." $LOG_APPEND $LOG_ERR
    die 15 "Unable to create destination install directory <$INST_DIR>"
  fi
  log "Created directory <$INST_DIR>." $LOG_APPEND

  $GUNZIP < $BASE_DIR/$BHN_LHD_SOURCE | $TAR xf - -C $INST_DIR
  if [ $? -ne 0 ]; then
    log "Failed to extract driver source from <$BASE_DIR/$BHN_LHD_SOURCE> to <$INST_DIR>." $LOG_APPEND $LOG_ERR
    die 19 "Unable to extract driver source code"
  fi
  log "Extracted driver source from <$BASE_DIR/$BHN_LHD_SOURCE> to <$INST_DIR>." $LOG_APPEND

  gen_uninst

  $AWK " {
if (/^KERNEL_DIR :=/) print \"KERNEL_DIR := $INST_KERNEL_BUILD\";
else print
} " < $INST_DIR/$BHN_LHD_DIR/Makefile > $INST_DIR/$BHN_LHD_DIR/Makefile.$$

  if [ $? -ne 0 ]; then
    log "Failed to update makefile with kernel build <$INST_KERNEL_BUILD>." $LOG_APPEND $LOG_ERR
    die 22 "Unable to update makefile kernel build env"
  fi
  log "Updated makefile with kernel build <$INST_KERNEL_BUILD>." $LOG_APPEND

  $MV $INST_DIR/$BHN_LHD_DIR/Makefile.$$ $INST_DIR/$BHN_LHD_DIR/Makefile
  if [ $? -ne 0 ]; then
    log "Failed to move temporary makefile." $LOG_APPEND $LOG_ERR
    die 20 "Unable to move temporary makefile"
  fi
}

#
# _commit_install_compile_source
#
_commit_install_compile_source() {

  log "Logging compile to <$INST_DIR/$BHN_LHD_DIR/mod_compile.log>." $LOG_APPEND
  $MAKE -C $INST_DIR/$BHN_LHD_DIR >$INST_DIR/$BHN_LHD_DIR/mod_compile.log 2>&1
  if [ $? -ne 0 ]; then
    log "Failed to build driver source in <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND $LOG_ERR
    die 7 "There was a problem compiling the driver"
  fi
  log "Built driver source in <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND
}

#
# _commit_install_install_driver_module
#
_commit_install_install_driver_module() {

  log "Logging install to <$INST_DIR/$BHN_LHD_DIR/mod_install.log>." $LOG_APPEND
  $MAKE -C $INST_DIR/$BHN_LHD_DIR install >$INST_DIR/$BHN_LHD_DIR/mod_install.log 2>&1
  if [ $? -ne 0 ]; then
    log "Failed to install driver module from <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND $LOG_ERR
    die 9 "There was a problem installing the driver module"
  fi
  log "Installed driver module from <$INST_DIR/$BHN_LHD_DIR>." $LOG_APPEND
}

#
# _commit_install_load_driver
#
_commit_install_load_driver() {
  local mod

  mod=`$LSMOD | $GREP "^bhn" | $CUT -d" " -f1`
  if [ $? -ne 0 ]; then
    log "Attempting to find already loaded <bhn> module failed." $LOG_APPEND $LOG_WARN
  fi
  if [ "$mod" != "" ] && [ "$mod" == "bhn" ]; then
    $RMMOD bhn
    if [ $? -ne 0 ]; then
      log "Failed to remove already loaded <bhn> module." $LOG_APPEND $LOG_WARN
    fi
  fi

  if [ $KERNEL_VERSION -eq 0 ]; then
    mod=`$LSMOD | $GREP "^ksocket_nosomaxconn" | $CUT -d" " -f1`
    if [ $? -ne 0 ]; then
      log "Attempting to find already loaded <ksocket_nosomaxconn> module failed." $LOG_APPEND $LOG_WARN
    fi
    if [ "$mod" != "" ] && [ "$mod" == "ksocket_nosomaxconn" ]; then
      $RMMOD ksocket_nosomaxconn
      if [ $? -ne 0 ]; then
        log "Failed to remove already loaded <ksocket_nosomaxconn> module." $LOG_APPEND $LOG_WARN
      fi
    fi
  fi

  $MODPROBE bhn
  if [ $? -ne 0 ]; then
    log "Failed to load (modprobe) driver module <bhn>." $LOG_APPEND $LOG_ERR
    die 10 "Unable to load module"
  fi
  log "Driver module <bhn> loaded (modprobe)." $LOG_APPEND
}

#
# _commit_install_autod
#
_commit_install_autod() {

  $CAT /proc/bhn/autodiscover > $INST_AUTOD_CONFIG_FILE
  if [ $? -ne 0 ]; then
    log "Failed to cat </proc/bhn/autodiscover> to <$INST_AUTOD_CONFIG_FILE>." $LOG_APPEND $LOG_ERR
    die 29 "Unable to generate config file <$INST_AUTOD_CONFIG_FILE> using autodiscovery"
  fi
  log "Config file <$INST_AUTOD_CONFIG_FILE> generated from </proc/bhn/autodiscover>." $LOG_APPEND
}

#
# _commit_install_config_driver
#
_commit_install_config_driver() {

  echo "clear-mappings" > /proc/bhn/config
  log "Cleared any old mappings." $LOG_APPEND
  $CAT $INST_CONFIG_FILE > /proc/bhn/config
  if [ $? -ne 0 ]; then
    log "Failed to cat <$INST_CONFIG_FILE> to </proc/bhn/config>." $LOG_APPEND $LOG_ERR
    die 30 "Unable to configure driver with <$INST_CONFIG_FILE>"
  fi
  log "Driver configured from <$INST_CONFIG_FILE>." $LOG_APPEND
}

#
# commit_install - performs the actual installation
#
commit_install() {

  if [ $CL_GCONF -eq 1 ]; then
    gen_conf
  fi

  echo -n "Installing driver source..."
  log "+++Installing driver source..." $LOG_APPEND
  _commit_install_install_source
  log "---finished installing driver source." $LOG_APPEND
  echo "done."

  echo -n "Compiling the driver..."
  log "+++Compiling driver..." $LOG_APPEND
  _commit_install_compile_source
  log "---finished compiling driver." $LOG_APPEND
  echo "done."

  echo -n "Installing the driver module..."
  log "+++Installing driver module..." $LOG_APPEND
  _commit_install_install_driver_module
  log "---finished installing driver module." $LOG_APPEND
  echo "done."

  echo -n "Adding module configuration..."
  log "+++Adding module configuration..." $LOG_APPEND
  eval "$DISTRO_ID""_modconf_commit"
  log "---finished adding module configuration." $LOG_APPEND
  echo "done."

  if [ $INST_KERNEL_RUNNING -eq 1 ]; then
    if [ $INST_LOAD_DRIVER -eq 1 ]; then
      echo -n "Loading driver module..."
      log "+++Loading driver module..." $LOG_APPEND
      _commit_install_load_driver
      log "---finished loading driver module." $LOG_APPEND
      echo "done."

      if [ $INST_AUTOD -eq 1 ]; then
        echo -n "Autodiscovering Blue Heat/Nets..."
        log "+++Autodiscovering Blue Heat/Nets..." $LOG_APPEND
        _commit_install_autod
        log "---finished autodiscovering Blue Heat/Nets." $LOG_APPEND
        echo "done."
      fi

      if [ $INST_CONFIG_DRIVER -eq 1 ]; then
        echo -n "Configuring the driver..."
        log "+++Configuring driver with <$INST_CONFIG_FILE>..." $LOG_APPEND
        _commit_install_config_driver
        log "---finished configuring driver with <$INST_CONFIG_FILE>." $LOG_APPEND
        echo "done."
      fi
    fi
  fi

  if [ $INST_INIT_SCRIPT -eq 1 ]; then
    echo -n "Generating $DISTRO_NAME init script..."
    log "+++Generating init script..." $LOG_APPEND
    eval "$DISTRO_ID""_init_script_commit"
    log "---finished generating init script." $LOG_APPEND
    echo "done."
  fi
}

#
# _do_config_install_check_distro - verifies distro information from
#                                   config file and local sys
#
_do_config_install_check_distro() {
  local config_distro_id
  local config_distro_name

  if [ "$DISTRO_ID" != "" ]; then
    distro_ID_to_NAME
  fi
  if [ "$DISTRO_NAME" != "" ]; then
    distro_NAME_to_ID
  fi

  validate_DISTRO
  if [ $RETVAL -eq 1 ]; then
    log "Configuration file specifies <$DISTRO_NAME> distribution." $LOG_APPEND
    config_distro_id="$DISTRO_ID"
    config_distro_name="$DISTRO_NAME"

    detect_DISTRO
    validate_DISTRO
    if [ $RETVAL -eq 1 ]; then
      log "Detected <$DISTRO_NAME> distribution." $LOG_APPEND
      if [ "$DISTRO_ID" != "$config_distro_id" ]; then
        if [ $CL_FORCE -eq 0 ]; then
          log "Distribution mismatch rejected." $LOG_APPEND $LOG_ERR
          die 13 "Distribution mismatch without force (config=<$config_distro_name>, detected=<$DISTRO_NAME>)"
        else
          log "Forcing <$config_distro_name> distribution." $LOG_APPEND $LOG_WARN
        fi
      fi
    else
      log "Could not automatically detect distribution." $LOG_APPEND $LOG_WARN
      if [ $CL_FORCE -eq 0 ]; then
        get_DISTRO
        log "User entered <$DISTRO_NAME> distribution." $LOG_APPEND
        if [ "$DISTRO_ID" != "$config_distro_id" ]; then
          log "Distribution mismatch rejected." $LOG_APPEND $LOG_ERR
          die 14 "Distribution mismatch (config=<$config_distro_name>, user entered=<$DISTRO_NAME>"
        fi
      fi
    fi

    DISTRO_ID="$config_distro_id"
    DISTRO_NAME="$config_distro_name"
  else
    log "Configuration file does not specify a distribution." $LOG_APPEND $LOG_WARN
    detect_DISTRO
    validate_DISTRO
    if [ $RETVAL -eq 0 ]; then
      log "Could not automatically detect distribution." $LOG_APPEND $LOG_WARN
      if [ $CL_FORCE -eq 1 ]; then
        die 11 "Could not determine distribution"
      else
        get_DISTRO
        log "User entered <$DISTRO_NAME> distribution." $LOG_APPEND
      fi
    else
      log "Detected <$DISTRO_NAME> distribution." $LOG_APPEND
    fi
  fi

  log "Installing as <$DISTRO_NAME> distribution." $LOG_APPEND
}

#
# _do_config_install_check_version - verifies config file version with
#                                    script version
#
_do_config_install_check_version() {

  if [ "$CONFIG_VERSION" = "$CONFIG_FILE_VERSION" ]; then
    log "Configuration file version <$CONFIG_FILE_VERSION> accepted." $LOG_APPEND
  else

    if [ $CL_FORCE -eq 1 ]; then
      log "Configuration file version <$CONFIG_FILE_VERSION> accepted by force." $LOG_APPEND $LOG_WARN
    else
      # we are allowed to prompt
      get_yesno "Version information in the specified configuration file ($CONFIG_FILE_VERSION)\ndoes not match that of this script ($CONFIG_VERSION).\nContinue anyways?" 0

      if [ $RETVAL -eq 1 ]; then
        # the user said yes
        log "Configuration file version <$CONFIG_FILE_VERSION> accepted by user." $LOG_APPEND
      else
        # the user said no
        log "Configuration file version <$CONFIG_FILE_VERSION> rejected by user." $LOG_APPEND $LOG_ERR
        die 8 "Configuration file version <$CONFIG_FILE_VERSION> was rejected by user"
      fi
    fi
  fi
}

#
# do_config_install - perform an installation based on a configuration file
#
do_config_install() {

  if [ -f "$CL_UCONFFILE" ]; then
    log "Using configuration file <$CL_UCONFFILE>." $LOG_APPEND
    source "$CL_UCONFFILE"
  else
    log "Could not find configuration file <$CL_UCONFFILE>." $LOG_APPEND $LOG_ERR
    die 3 "Specified configuration file not found ($CL_UCONFFILE)"
  fi

  _do_config_install_check_version
  _do_config_install_check_distro

  eval "$DISTRO_ID""_inst_requirements"

  commit_install
}

#
# _do_install_intro - display intoduction and wait for input from user
#
_do_install_intro() {
  local response

  echo
  echo "Welcome to the Blue Heat/Net Linux Host Driver install script."
  echo "You will be asked a few short questions about your installation."
  echo "There is a final confirmation before proceeding with the"
  echo "installation and you may exit at any time by pressing <CTRL-C>."
  echo
  echo "Questions will have their default value shown in []s.  This default"
  echo "value will be used if no value is specified."
  echo
  echo "Press <ENTER> to continue"
  read response
}

#
# _do_install_get_inst_dir - get installation directory
#
_do_install_get_inst_dir() {
  local finished=0

  while [ $finished -ne 1 ]; do
    echo
    get_path "Where would you like to install the driver?" "$DEFAULT_INST_DIR" $PATH_DIR $PATH_MAY_EXIST $PATH_NO_CONFIRM_OVERW
    INST_DIR="`make_abs_path "$RETVAL" "$START_DIR"`"

    if [ -d "$INST_DIR/$BHN_LHD_DIR" ]; then
      get_yesno "WARNING: Overwrite existing installation?" 0
      if [ $RETVAL -eq 1 ]; then
        finished=1
      fi
    else
      finished=1
    fi
  done
  log "[user] Selected <$INST_DIR> install directory." $LOG_APPEND
}

#
# _do_install_get_kern_info - get kernel information
#
_do_install_get_kern_info() {
  local old_dir
  local i
  local kernels=""
  local -i finished=0
  local response

  echo
  if [ -d "$KERNEL_BUILDS" ]; then
    old_dir="$PWD"
    cd $KERNEL_BUILDS
    for i in `$LS`; do
      if [ -f "$i/build/include/linux/autoconf.h" ] &&
                      [ -f "$i/build/include/linux/version.h" ]; then
         kernels="$kernels $i/build"
      fi
    done
    cd $old_dir
  fi

  echo "The following kernels are currently available:"
  if [ "$kernels" = "" ]; then
    echo "<no kernels found>"
  else
    for i in $kernels; do
      echo "$KERNEL_BUILDS/$i"
    done
    INST_KERNEL_BUILD="$KERNEL_BUILDS/""`echo $kernels | $AWK '{print $1}'`"
  fi
  echo
  echo "If you do not see the kernel you wish to use listed, it likely has not"
  echo "been compiled yet. Please exit <CTRL-C> and do that first."

  while [ $finished -ne 1 ]; do
    echo
    get_path "Which kernel source tree do you wish to compile against?" "$INST_KERNEL_BUILD" $PATH_DIR $PATH_MUST_EXIST $PATH_NO_CONFIRM_OVERW
    INST_KERNEL_BUILD=`make_abs_path "$RETVAL" "$START_DIR"`
    if [ ! -f "$INST_KERNEL_BUILD/include/linux/autoconf.h" ] ||
                    [ ! -f "$INST_KERNEL_BUILD/include/linux/version.h" ]; then
      echo "Could not find kernel build environment at "$INST_KERNEL_BUILD", please make another selection."
    else
      finished=1
      get_yesno "Is this the kernel currently running?" $DEFAULT_INST_KERNEL_RUNNING
      INST_KERNEL_RUNNING=$RETVAL
    fi
  done

  log "[user] Selected <$INST_KERNEL_BUILD> kernel build environment." $LOG_APPEND
  log "[user] Confirmed running kernel with <$INST_KERNEL_RUNNING>." $LOG_APPEND
}

#
# _do_install_get_load_driver
#
_do_install_get_load_driver() {
  echo
  echo "To link the Blue Heat/Net to local /dev nodes you must configure"
  echo "which Blue Heat/Net port attaches to which /dev/bhnS* node.  To"
  echo "facilitate this configuration, an autodiscovery tool is provided."
  echo "To use the autodiscovery tool, the driver must first be loaded."
  echo
  get_yesno "Would you like to load (modprobe) the driver after installing it?" $INST_LOAD_DRIVER
  INST_LOAD_DRIVER=$RETVAL
  log "[user] Confirmed driver load with <$INST_LOAD_DRIVER>." $LOG_APPEND
}

#
# _do_install_get_autod
#
_do_install_get_autod() {
  echo
  echo "The autodiscovery tool searches the network for any Blue Heat/Nets."
  echo "This tool can be used to generate a configuration file for use with"
  echo "the Blue Heat/Net driver."
  echo "It is recommended you save your configuration to a local /etc file."
  echo
  get_yesno "Autodiscover all Blue Heat/Nets on your network?" $INST_AUTOD
  INST_AUTOD=$RETVAL
  log "[user] Confirmed autodiscover with <$INST_AUTOD>." $LOG_APPEND

  if [ $INST_AUTOD -eq 1 ]; then
    get_path "Enter the location to store your configuration file:" "$INST_AUTOD_CONFIG_FILE" $PATH_FILE $PATH_MAY_EXIST $PATH_CONFIRM_OVERW
    INST_AUTOD_CONFIG_FILE="`make_abs_path "$RETVAL" "$START_DIR"`"
    log "[user] Storing autodiscovery config to <$INST_AUTOD_CONFIG_FILE>." $LOG_APPEND
  fi
}

#
# _do_install_get_config_driver
#
_do_install_get_config_driver() {
  echo
  echo "After installing and loading the driver, the driver will not be"
  echo "configured.  Configuring the driver maps Blue Heat/Net ports"
  echo "to virtual device nodes on the local system."
  echo
  get_yesno "Do you wish to configure the driver after loading?" $INST_CONFIG_DRIVER
  INST_CONFIG_DRIVER=$RETVAL
  log "[user] Confirmed config driver with <$INST_CONFIG_DRIVER>." $LOG_APPEND

  if [ $INST_CONFIG_DRIVER -eq 1 ]; then
    get_path "Enter the location of the configuration file to use:" "$INST_CONFIG_FILE" $PATH_FILE $PATH_MAY_EXIST $PATH_NO_CONFIRM_OVERW
    INST_CONFIG_FILE="`make_abs_path "$RETVAL" "$START_DIR"`"
    log "[user] Configuring driver with <$INST_CONFIG_FILE>." $LOG_APPEND
  fi
}

#
# _do_install_other_init
#
_do_install_other_init() {
  local response

  log "Cannot install init script because distro is not supported." $LOG_APPEND $LOG_WARN

  echo
  echo "This distribution is not supported and an initialization script"
  echo "cannot be automatically installed."
  echo "To inquire about support for your distribution, please contact"
  echo "Connect Tech technical support at support@connecttech.com"
  echo
  echo "Press <ENTER> to continue"
  read response
  INST_INIT_SCRIPT=0
}

#
# _do_install_get_init
#
_do_install_get_init() {
  echo
  echo "An initialization script for your system can be created.  This"
  echo "initialization script allows the Blue Heat/Net driver to be loaded"
  echo "automatically during system startup."
  echo
  get_yesno "Install initialization script into system?" $INST_INIT_SCRIPT
  INST_INIT_SCRIPT=$RETVAL
  log "[user] Confirmed init script install with <$INST_INIT_SCRIPT>." $LOG_APPEND

  if [ $INST_INIT_SCRIPT -eq 1 ]; then
    echo
    echo "The initialization script requires a configuration file in order to configure the driver."
    echo
    get_path "Enter the location of the configuration file to use:" "$INST_INIT_SCRIPT_CONFIG_FILE" $PATH_FILE $PATH_MAY_EXIST $PATH_NO_CONFIRM_OVERW
    INST_INIT_SCRIPT_CONFIG_FILE="`make_abs_path "$RETVAL" "$START_DIR"`"
    log "[user] Generated init script will use <$INST_INIT_SCRIPT_CONFIG_FILE>." $LOG_APPEND
  fi
}

#
# _do_install_confirm
#
_do_install_confirm() {
  local kernel_running_str
  local load_driver_str
  local autod_str
  local config_driver_str
  local init_script_str

  if [ $INST_KERNEL_RUNNING -eq 1 ]; then
    kernel_running_str="Yes"
  else
    kernel_running_str="No"
  fi
  if [ $INST_LOAD_DRIVER -eq 1 ]; then
    load_driver_str="Yes"
  else
    load_driver_str="No"
  fi
  if [ $INST_AUTOD -eq 1 ]; then
    autod_str="Yes"
  else
    autod_str="No"
  fi
  if [ $INST_CONFIG_DRIVER -eq 1 ]; then
    config_driver_str="Yes"
  else
    config_driver_str="No"
  fi
  if [ $INST_INIT_SCRIPT -eq 1 ]; then
    init_script_str="Yes"
  else
    init_script_str="No"
  fi

  echo
  echo "Using distribution [$DISTRO_NAME]"
  echo "Install driver source to [$INST_DIR]"
  eval "$DISTRO_ID""_modconf_display"
  echo "Compile driver using kernel build [$INST_KERNEL_BUILD]"
  echo "Compiling for active kernel [$kernel_running_str]"

  if [ $INST_KERNEL_RUNNING -eq 1 ]; then
    echo "Load driver (modprobe) after install [$load_driver_str]"
    if [ $INST_LOAD_DRIVER -eq 1 ]; then
      echo "  +Generate config file using autodiscovery [$autod_str]"
      if [ $INST_AUTOD -eq 1 ]; then
        echo "    (Saved to [$INST_AUTOD_CONFIG_FILE])"
        echo "    At this point it is important that your Blue Heat/Net is powered"
        echo "    and running on the network.  To check this, switch to a different"
        echo "    terminal and try to ping the Blue Heat/Net."
      fi
      echo "  +Configure driver after load [$config_driver_str]"
      if [ $INST_CONFIG_DRIVER -eq 1 ]; then
        echo "    (Using [$INST_CONFIG_FILE])"
      fi
    fi
  fi

  echo "Install initialization script [$init_script_str]"
  if [ $INST_INIT_SCRIPT -eq 1 ]; then
    echo "  ($DISTRO_NAME style init script using [$INST_INIT_SCRIPT_CONFIG_FILE])"
  fi

  echo
  get_yesno "Proceed with install?" 0
  log "[user] Confirmed installation parameters with <$RETVAL>." $LOG_APPEND
  if [ $RETVAL -eq 1 ]; then
#    clear
    commit_install
  fi
}

#
# do_install - perform an installation in interactive mode
#
do_install() {

#  clear
  _do_install_intro

  detect_DISTRO
  validate_DISTRO
  if [ $RETVAL -eq 1 ]; then
    log "Detected <$DISTRO_NAME> distribution." $LOG_APPEND
  else
    log "Could not detect distribution." $LOG_APPEND $LOG_WARN
#    clear
    get_DISTRO
    log "[user] Selected <$DISTRO_NAME> distribution." $LOG_APPEND
  fi

  eval "$DISTRO_ID""_inst_requirements"

#  clear
  _do_install_get_inst_dir

#  clear
  _do_install_get_kern_info

#  clear
  eval "$DISTRO_ID""_modconf_setup"

  if [ $INST_KERNEL_RUNNING -eq 1 ]; then

#    clear
    _do_install_get_load_driver

    if [ $INST_LOAD_DRIVER -eq 1 ]; then

#      clear
      _do_install_get_autod
      if [ $INST_AUTOD -eq 1 ]; then
        INST_CONFIG_FILE="$INST_AUTOD_CONFIG_FILE"
        INST_INIT_SCRIPT_CONFIG_FILE="$INST_AUTOD_CONFIG_FILE"
      fi

#      clear
      _do_install_get_config_driver
      if [ $INST_CONFIG_DRIVER -eq 1 ]; then
        INST_INIT_SCRIPT_CONFIG_FILE="$INST_CONFIG_FILE"
      fi
    fi
  fi

  if [ "$DISTRO_ID" = "$DISTRO_OTHER_ID" ]; then
#    clear
    _do_install_other_init
  else
#    clear
    _do_install_get_init
  fi

#  clear
  _do_install_confirm
}

#
# Main
#

which 2> /dev/null
if [ $? -eq 127 ]; then
  die 2 "Required utility 'which' not found"
fi

check_requirements

THIS=`$BASENAME $THIS`
START_DIR="$PWD"
BASE_DIR="`$DIRNAME $0`"
BASE_DIR="`make_abs_path "$BASE_DIR" "$START_DIR"`"

source $BASE_DIR/utils
source $BASE_DIR/distros

# process command line with $@ instead of $* because $* mushes the parameters
# into one word, $@ keeps them separate
parse_command_line "$@"

# this script only supports Linux
check_linux

# only root may run this
check_root

require "file" "$BASE_DIR/$BHN_LHD_SOURCE"

trap do_exit $SIG_MASK

log "Starting Blue Heat/Net install script." $LOG_NO_APPEND

if [ $CL_UCONF -eq 1 ]; then
  do_config_install
else
  do_install
fi

SIG_EXIT=0
do_exit
